@@ -23,6 +23,7 @@
 #include <folly/ScopeGuard.h>
 #include <folly/Traits.h>
 #include <folly/functional/Invoke.h>
+#include <folly/lang/Hint.h>
 #include <folly/portability/GFlags.h>
 
 #include <cassert>
@@ -36,6 +37,7 @@
 #include <glog/logging.h>
 
 DECLARE_bool(benchmark);
+DECLARE_uint32(bm_result_width_chars);
 
 namespace folly {
 
@@ -363,7 +365,7 @@ void printResultComparison(
  * Allow users to record customized counter during benchmarking,
  * there will be one extra column showing in the output result for each counter
  *
- * BENCHMARK_COUNTERS(insertVectorBegin, couters, iters) {
+ * BENCHMARK_COUNTERS(insertVectorBegin, counters, iters) {
  *   vector<int> v;
  *   FOR_EACH_RANGE (i, 0, iters) {
  *     v.insert(v.begin(), 42);
