@@ -30,7 +30,7 @@
 static_assert(FOLLY_CPLUSPLUS >= 201402L, "__cplusplus >= 201402L");
 
 #if defined(__GNUC__) && !defined(__clang__)
-static_assert(__GNUC__ >= 7, "__GNUC__ >= 5");
+static_assert(__GNUC__ >= 7, "__GNUC__ >= 7");
 #endif
 
 // Unaligned loads and stores
@@ -633,3 +633,10 @@ constexpr auto kCpplibVer = 0;
 #else
 #define FOLLY_HAS_STRING_VIEW 0
 #endif
+
+// C++20 consteval
+#if FOLLY_CPLUSPLUS >= 202002L
+#define FOLLY_CONSTEVAL consteval
+#else
+#define FOLLY_CONSTEVAL constexpr
+#endif
