@@ -388,8 +388,6 @@ struct SingletonHolder : public SingletonHolderBase {
 
 class SingletonVault {
  public:
-  static void atfork_init();
-
   enum class Type {
     Strict, // Singletons can't be created before registrationComplete()
     Relaxed, // Singletons can be created before registrationComplete()
@@ -439,6 +437,8 @@ class SingletonVault {
    */
   void addEagerInitSingleton(detail::SingletonHolderBase* entry);
 
+  void addEagerInitOnReenableSingleton(detail::SingletonHolderBase* entry);
+
   // Mark registration is complete; no more singletons can be
   // registered at this point.
   void registrationComplete();
@@ -564,8 +564,16 @@ class SingletonVault {
       std::unordered_set<detail::SingletonHolderBase*>,
       SharedMutexSuppressTSAN>
       eagerInitSingletons_;
+  Synchronized<
+      std::unordered_set<detail::SingletonHolderBase*>,
+      SharedMutexSuppressTSAN>
+      eagerInitOnReenableSingletons_;
   Synchronized<std::vector<detail::TypeDescriptor>, SharedMutexSuppressTSAN>
       creationOrder_;
+  Synchronized<
+      std::unordered_set<detail::TypeDescriptor, detail::TypeDescriptorHasher>,
+      SharedMutexSuppressTSAN>
+      instantiatedAtLeastOnce_;
   std::unordered_set<detail::SingletonHolderBase*> liveSingletonsPreFork_;
 
   // Using SharedMutexReadPriority is important here, because we want to make
@@ -581,11 +589,15 @@ class SingletonVault {
   bool failOnUseAfterFork_{true};
 };
 
-// This is the wrapper class that most users actually interact with.
-// It allows for simple access to registering and instantiating
-// singletons.  Create instances of this class in the global scope of
-// type Singleton<T> to register your singleton for later access via
-// Singleton<T>::try_get().
+/**
+ * Singleton allows for simple access to registering and instantiating
+ * singletons.  Create instances of this class in the global scope of
+ * type Singleton<T> to register your singleton for later access via
+ * Singleton<T>::try_get().
+ *
+ * There are many supporting libraries and classes for Singleton; this is the
+ * one that users typically interact with.
+ */
 template <
     typename T,
     typename Tag = detail::DefaultTag,
@@ -595,38 +607,55 @@ class Singleton {
   typedef std::function<T*(void)> CreateFunc;
   typedef std::function<void(T*)> TeardownFunc;
 
-  // Generally your program life cycle should be fine with calling
-  // get() repeatedly rather than saving the reference, and then not
-  // call get() during process shutdown.
+  /**
+   * Get a pointer to the singleton.
+   *
+   * It is preferable to call get() repeatedly than to store the returned
+   * pointer. Accessing the returned pointer often fails during shutdown.
+   *
+   * Deprecated in favor of try_get().
+   */
   [[deprecated("Replaced by try_get")]] static T* get() {
     return getEntry().get();
   }
 
-  // If, however, you do need to hold a reference to the specific
-  // singleton, you can try to do so with a weak_ptr.  Avoid this when
-  // possible but the inability to lock the weak pointer can be a
-  // signal that the vault has been destroyed.
+  /**
+   * Get a weak_ptr to the singleton.
+   *
+   * If you cannot lock the weak_ptr, this usually means the vault has been
+   * destroyed.
+   *
+   * Deprecated in favor of try_get().
+   */
   [[deprecated("Replaced by try_get")]] static std::weak_ptr<T> get_weak() {
     return getEntry().get_weak();
   }
 
-  // Preferred alternative to get_weak, it returns shared_ptr that can be
-  // stored; a singleton won't be destroyed unless shared_ptr is destroyed.
-  // Avoid holding these shared_ptrs beyond the scope of a function;
-  // don't put them in member variables, always use try_get() instead
-  //
-  // try_get() can return nullptr if the singleton was destroyed, caller is
-  // responsible for handling nullptr return
+  /**
+   * Get a shared_ptr to the singleton.
+   *
+   * It is recommended to call try_get() repeatedly, rather than storing the
+   * shared_ptr, because storing the shared_ptr prevents the singleton from
+   * being destroyed during shutdown.
+   */
   static std::shared_ptr<T> try_get() { return getEntry().try_get(); }
 
+  /**
+   * Get a ReadMostlySharedPtr to the singleton.
+   *
+   * It is recommended to call try_get_fast() repeatedly, rather than storing
+   * the ReadMostlySharedPtr, because storing the ReadMostlySharedPtr prevents
+   * the singleton from being destroyed during shutdown.
+   */
   static folly::ReadMostlySharedPtr<T> try_get_fast() {
     return getEntry().try_get_fast();
   }
 
   /**
    * Applies a callback to the possibly-nullptr singleton instance, returning
-   * the callback's result. That is, the following two are functionally
-   * equivalent:
+   * the callback's result.
+   *
+   * That is, the following two are functionally equivalent:
    *    singleton.apply(std::ref(f));
    *    f(singleton.try_get().get());
    *
@@ -639,14 +668,24 @@ class Singleton {
     return getEntry().apply(std::ref(f));
   }
 
-  // Quickly ensure the instance exists.
+  /// Ensure the instance exists.
   static void vivify() { getEntry().vivify(); }
 
+  /**
+   * Create a singleton, which uses the default-constructor for its wrapped
+   * type.
+   *
+   * @param t  The teardown function to use (in lieu of delete).
+   */
   explicit Singleton(
       std::nullptr_t /* _ */ = nullptr,
       typename Singleton::TeardownFunc t = nullptr)
       : Singleton([]() { return new T; }, std::move(t)) {}
 
+  /**
+   * @param c  The create function to use (in lieu of new).
+   * @param t  The teardown function to use (in lieu of delete).
+   */
   explicit Singleton(
       typename Singleton::CreateFunc c,
       typename Singleton::TeardownFunc t = nullptr) {
@@ -660,6 +699,8 @@ class Singleton {
   }
 
   /**
+   * Specify that the singleton should be eagerly initialized.
+   *
    * Should be instantiated as soon as "doEagerInit[Via]" is called.
    * Singletons are usually lazy-loaded (built on-demand) but for those which
    * are known to be needed, to avoid the potential lag for objects that take
@@ -667,7 +708,7 @@ class Singleton {
    * are built up-front.
    *
    * Use like:
-   *   Singleton<Foo> gFooInstance = Singleton<Foo>(...).shouldEagerInit();
+   *   auto gFooInstance = Singleton<Foo>(...).shouldEagerInit();
    *
    * Or alternately, define the singleton as usual, and say
    *   gFooInstance.shouldEagerInit();
@@ -682,6 +723,25 @@ class Singleton {
   }
 
   /**
+   * Specify that the singleton should be eagerly initialized when singletons
+   * reenable.
+   *
+   * Should be re-instantiated as soon as reenableInstances() is called.
+   * Note that it will be re-instantiated only if it was instantiated before
+   * destroyInstances() call.
+   *
+   * Use like:
+   *   auto gFooInstance = Singleton<Foo>(...).shouldEagerInitOnReenable();
+   */
+  Singleton& shouldEagerInitOnReenable() {
+    auto vault = SingletonVault::singleton<VaultTag>();
+    vault->addEagerInitOnReenableSingleton(&getEntry());
+    return *this;
+  }
+
+  /**
+   * Inject a mock singleton, for testing.
+   *
    * Construct and inject a mock singleton which should be used only from tests.
    * Unlike regular singletons which are initialized once per process lifetime,
    * mock singletons live for the duration of a test. This means that one
