@@ -195,7 +195,7 @@ class TransformProcessorBase : public IChannelCallback {
       values.pop();
       bool inputClosed = !inputResult.hasValue();
       if (!inputResult.hasValue() && !inputResult.hasException()) {
-        inputResult = folly::Try<InputValueType>(OnClosedException());
+        inputResult = Try<InputValueType>(OnClosedException());
       }
       auto outputGen = folly::makeTryWith([&]() {
         return transformer_.transformValue(std::move(inputResult));
@@ -427,7 +427,7 @@ class ResumableTransformProcessor : public TransformProcessorBase<
         auto* reinitializeEx =
             closeResult.exception
                 ->get_exception<ReinitializeException<InitializeArg>>();
-        co_await initializeImpl(reinitializeEx->initializeArg);
+        co_await initializeImpl(std::move(reinitializeEx->initializeArg));
         co_return;
       }
     }
@@ -479,7 +479,7 @@ class DefaultTransformer : public RateLimiterHolder<RateLimiterEnabled> {
     return executor_;
   }
 
-  auto transformValue(folly::Try<InputValueType> inputValue) {
+  auto transformValue(Try<InputValueType> inputValue) {
     return transformValue_(std::move(inputValue));
   }
 
