@@ -23,9 +23,6 @@
 #include <folly/Optional.h>
 #include <folly/Portability.h>
 #include <folly/ScopeGuard.h>
-#ifdef __APPLE__
-#include <folly/ThreadLocal.h>
-#endif
 #include <folly/Try.h>
 #include <folly/fibers/Baton.h>
 #include <folly/fibers/Fiber.h>
@@ -80,7 +77,17 @@ inline void FiberManager::activateFiber(Fiber* fiber) {
 #endif
 
   activeFiber_ = fiber;
+
+#ifdef FOLLY_SANITIZE_THREAD
+  auto tsanCtx = __tsan_get_current_fiber();
+  __tsan_switch_to_fiber(fiber->tsanCtx_, 0);
+#endif
+
   fiber->fiberImpl_.activate();
+
+#ifdef FOLLY_SANITIZE_THREAD
+  __tsan_switch_to_fiber(tsanCtx, 0);
+#endif
 }
 
 inline void FiberManager::deactivateFiber(Fiber* fiber) {
@@ -151,8 +158,8 @@ inline void FiberManager::runReadyFiber(Fiber* fiber) {
     fiber->rcontext_ = RequestContext::saveContext();
     fiber->asyncRoot_ = folly::exchangeCurrentAsyncStackRoot(nullptr);
   } else if (fiber->state_ == Fiber::INVALID) {
-    assert(fibersActive_ > 0);
-    --fibersActive_;
+    assert(fibersActive_.load(std::memory_order_relaxed) > 0);
+    fibersActive_.fetch_sub(1, std::memory_order_relaxed);
     // Making sure that task functor is deleted once task is complete.
     // NOTE: we must do it on main context, as the fiber is not
     // running at this point.
@@ -235,11 +242,6 @@ void FiberManager::runFibersHelper(LoopFunc&& loopFunc) {
     CHECK(oldAsyncRoot == nullptr);
 
     yieldedFibers_ = prevYieldedFibers;
-    if (observer_) {
-      for (auto& yielded : yieldedFibers) {
-        observer_->runnable(reinterpret_cast<uintptr_t>(&yielded));
-      }
-    }
     readyFibers_.splice(readyFibers_.end(), yieldedFibers);
     RequestContext::setContext(std::move(curCtx));
     if (!readyFibers_.empty()) {
@@ -287,9 +289,6 @@ inline void FiberManager::loopUntilNoReadyImpl() {
             fiber->rcontext_ = std::move(task->rcontext);
 
             fiber->setFunction(std::move(task->func), TaskOptions());
-            if (observer_) {
-              observer_->runnable(reinterpret_cast<uintptr_t>(fiber));
-            }
             runReadyFiber(fiber);
           });
 
@@ -375,10 +374,6 @@ Fiber* FiberManager::createTask(F&& func, TaskOptions taskOptions) {
     fiber->setFunction(std::ref(*funcLoc), std::move(taskOptions));
   }
 
-  if (observer_) {
-    observer_->runnable(reinterpret_cast<uintptr_t>(fiber));
-  }
-
   return fiber;
 }
 
@@ -519,10 +514,6 @@ Fiber* FiberManager::createTaskFinally(F&& func, G&& finally) {
     fiber->setFunctionFinally(std::ref(*funcLoc), std::ref(*finallyLoc));
   }
 
-  if (observer_) {
-    observer_->runnable(reinterpret_cast<uintptr_t>(fiber));
-  }
-
   return fiber;
 }
 
@@ -598,13 +589,8 @@ T& FiberManager::local() {
 
 template <typename T>
 T& FiberManager::localThread() {
-#ifndef __APPLE__
   static thread_local T t;
   return t;
-#else // osx doesn't support thread_local
-  static ThreadLocal<T> t;
-  return *t;
-#endif
 }
 
 inline void FiberManager::initLocalData(Fiber& fiber) {
@@ -623,18 +609,7 @@ FiberManager::FiberManager(
     : loopController_(std::move(loopController__)),
       stackAllocator_(options.guardPagesPerStack),
       options_(preprocessOptions(std::move(options))),
-      exceptionCallback_([](std::exception_ptr eptr, std::string context) {
-        try {
-          std::rethrow_exception(eptr);
-        } catch (const std::exception& e) {
-          LOG(DFATAL) << "Exception " << typeid(e).name() << " with message '"
-                      << e.what() << "' was thrown in "
-                      << "FiberManager with context '" << context << "'";
-        } catch (...) {
-          LOG(DFATAL) << "Unknown exception was thrown in FiberManager with "
-                      << "context '" << context << "'";
-        }
-      }),
+      exceptionCallback_(defaultExceptionCallback),
       fibersPoolResizer_(*this),
       localType_(typeid(LocalT)) {
   loopController_->setFiberManager(this);
