@@ -34,21 +34,13 @@
 #include <folly/futures/detail/Core.h>
 #include <folly/lang/Pretty.h>
 
-#if defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
-#include <folly/synchronization/Baton.h>
-#endif
-
 namespace folly {
 
 class Timekeeper;
 
 namespace futures {
 namespace detail {
-#if defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
-typedef folly::Baton<> FutureBatonType;
-#else
 typedef folly::fibers::Baton FutureBatonType;
-#endif
 } // namespace detail
 } // namespace futures
 
@@ -470,17 +462,13 @@ class WaitExecutor final : public folly::Executor {
   void drive() {
     baton_.wait();
 
-#if !defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
     fibers::runInMainContext([&]() {
-#endif
       baton_.reset();
       auto funcs = std::move(queue_.wlock()->funcs);
       for (auto& func : funcs) {
         std::exchange(func, nullptr)();
       }
-#if !defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
     });
-#endif
   }
 
   using Clock = std::chrono::steady_clock;
@@ -489,18 +477,14 @@ class WaitExecutor final : public folly::Executor {
     if (!baton_.try_wait_until(deadline)) {
       return false;
     }
-#if !defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
     return fibers::runInMainContext([&]() {
-#endif
       baton_.reset();
       auto funcs = std::move(queue_.wlock()->funcs);
       for (auto& func : funcs) {
         std::exchange(func, nullptr)();
       }
       return true;
-#if !defined(FOLLY_DISABLE_FUTURE_FIBERS_BATON)
     });
-#endif
   }
 
   void detach() {
@@ -827,6 +811,16 @@ SemiFuture<T> SemiFuture<T>::deferError(F&& func) && {
       });
 }
 
+template <class T>
+template <class F>
+SemiFuture<T> SemiFuture<T>::deferEnsure(F&& func) && {
+  return std::move(*this).defer(
+      [func = static_cast<F&&>(func)](Try<T>&& t) mutable {
+        static_cast<F&&>(func)();
+        return makeSemiFuture<T>(std::move(t));
+      });
+}
+
 template <class T>
 SemiFuture<Unit> SemiFuture<T>::unit() && {
   return std::move(*this).deferValue([](T&&) {});
@@ -1831,10 +1825,10 @@ std::vector<Future<Result>> window(
 // reduce
 
 template <class T>
-template <class I, class F>
-Future<I> Future<T>::reduce(I&& initial, F&& func) && {
+template <class In, class F>
+Future<In> Future<T>::reduce(In&& initial, F&& func) && {
   return std::move(*this).thenValue(
-      [minitial = static_cast<I&&>(initial),
+      [minitial = static_cast<In&&>(initial),
        mfunc = static_cast<F&&>(func)](T&& vals) mutable {
         auto ret = std::move(minitial);
         for (auto& val : vals) {
