@@ -132,8 +132,12 @@ void AsyncUDPSocket::init(sa_family_t family, BindOptions bindOptions) {
   if (reusePort_) {
     // put the socket in port reuse mode
     int value = 1;
-    if (netops::setsockopt(
-            socket, SOL_SOCKET, SO_REUSEPORT, &value, sizeof(value)) != 0) {
+    auto opt = SO_REUSEPORT;
+#ifdef _WIN32
+    opt = SO_BROADCAST;
+#endif
+    if (netops::setsockopt(socket, SOL_SOCKET, opt, &value, sizeof(value)) !=
+        0) {
       throw AsyncSocketException(
           AsyncSocketException::NOT_OPEN,
           "failed to put socket in reuse_port mode",
@@ -356,13 +360,19 @@ void AsyncUDPSocket::setDFAndTurnOffPMTU() {
 #if defined(IPV6_MTU_DISCOVER) && defined(IPV6_PMTUDISC_PROBE)
   optname6 = IPV6_MTU_DISCOVER;
   optval6 = IPV6_PMTUDISC_PROBE;
+#endif
+#if defined(_WIN32) && defined(IP_DONTFRAGMENT) && defined(IPV6_DONTFRAG)
+  optname4 = IP_DONTFRAGMENT;
+  optval4 = TRUE;
+  optname6 = IPV6_DONTFRAG;
+  optval6 = TRUE;
 #endif
   if (optname4 && optval4 && address().getFamily() == AF_INET) {
     if (folly::netops::setsockopt(
             fd_, IPPROTO_IP, optname4, &optval4, sizeof(optval4))) {
       throw AsyncSocketException(
           AsyncSocketException::NOT_OPEN,
-          "Failed to set PMTUDISC_PROBE with IP_MTU_DISCOVER",
+          "Failed to turn off fragmentation and PMTU discovery (IPv4)",
           errno);
     }
   }
@@ -371,7 +381,7 @@ void AsyncUDPSocket::setDFAndTurnOffPMTU() {
             fd_, IPPROTO_IPV6, optname6, &optval6, sizeof(optval6))) {
       throw AsyncSocketException(
           AsyncSocketException::NOT_OPEN,
-          "Failed to set PMTUDISC_PROBE with IPV6_MTU_DISCOVER",
+          "Failed to turn off fragmentation and PMTU discovery (IPv6)",
           errno);
     }
   }
