@@ -17,6 +17,7 @@
 #pragma once
 
 #include <memory>
+#include <folly/io/SocketOptionMap.h>
 
 #include <folly/Function.h>
 #include <folly/ScopeGuard.h>
@@ -235,8 +236,12 @@ class AsyncUDPSocket : public EventHandler {
    * Set extra control messages to send
    */
   virtual void setCmsgs(const SocketOptionMap& cmsgs);
+  virtual void setNontrivialCmsgs(
+      const SocketNontrivialOptionMap& nontrivialCmsgs);
 
   virtual void appendCmsgs(const SocketOptionMap& cmsgs);
+  virtual void appendNontrivialCmsgs(
+      const SocketNontrivialOptionMap& nontrivialCmsgs);
 
   /**
    * Send the data in buffer to destination. Returns the return code from
@@ -342,6 +347,11 @@ class AsyncUDPSocket : public EventHandler {
    */
   virtual void setFreeBind(bool freeBind) { freeBind_ = freeBind; }
 
+  /**
+   * Set IP_TRANSPARENT to allow enables transparent proxying on the socket
+   */
+  virtual void setTransparent(bool transparent) { transparent_ = transparent; }
+
   /**
    * Set reuse port mode to call bind() on the same address multiple times
    */
@@ -461,6 +471,9 @@ class AsyncUDPSocket : public EventHandler {
   virtual void applyOptions(
       const SocketOptionMap& options, SocketOptionKey::ApplyPos pos);
 
+  virtual void applyNontrivialOptions(
+      const SocketNontrivialOptionMap& options, SocketOptionKey::ApplyPos pos);
+
   /**
    * Override netops::Dispatcher to be used for netops:: calls.
    *
@@ -558,6 +571,7 @@ class AsyncUDPSocket : public EventHandler {
   bool reuseAddr_{false};
   bool reusePort_{false};
   bool freeBind_{false};
+  bool transparent_{false};
   int rcvBuf_{0};
   int sndBuf_{0};
   int busyPollUs_{0};
@@ -601,6 +615,8 @@ class AsyncUDPSocket : public EventHandler {
 
   SocketOptionMap cmsgs_;
 
+  SocketNontrivialOptionMap nontrivialCmsgs_;
+
   netops::DispatcherContainer netops_;
 };
 
