@@ -20,14 +20,18 @@
 
 namespace folly {
 
-const char* kTestCert = "folly/io/async/test/certs/tests-cert.pem";
-const char* kTestKey = "folly/io/async/test/certs/tests-key.pem";
-const char* kTestCA = "folly/io/async/test/certs/ca-cert.pem";
+#if !defined(FOLLY_CERTS_DIR)
+#define FOLLY_CERTS_DIR "folly/io/async/test/certs"
+#endif
+
+const char* kTestCert = FOLLY_CERTS_DIR "/tests-cert.pem";
+const char* kTestKey = FOLLY_CERTS_DIR "/tests-key.pem";
+const char* kTestCA = FOLLY_CERTS_DIR "/ca-cert.pem";
 const char* kTestCertCN = "Asox Company";
 
-const char* kClientTestCert = "folly/io/async/test/certs/client_cert.pem";
-const char* kClientTestKey = "folly/io/async/test/certs/client_key.pem";
-const char* kClientTestCA = "folly/io/async/test/certs/client_ca_cert.pem";
+const char* kClientTestCert = FOLLY_CERTS_DIR "/client_cert.pem";
+const char* kClientTestKey = FOLLY_CERTS_DIR "/client_key.pem";
+const char* kClientTestCA = FOLLY_CERTS_DIR "/client_ca_cert.pem";
 
 TestSSLServer::~TestSSLServer() {
   if (thread_.joinable()) {
