@@ -57,7 +57,7 @@
 #include <folly/memory/Malloc.h>
 #include <folly/portability/Malloc.h>
 
-#if (FOLLY_X64 || FOLLY_PPC64)
+#if (FOLLY_X64 || FOLLY_PPC64 || FOLLY_AARCH64)
 #define FOLLY_SV_PACK_ATTR FOLLY_PACK_ATTR
 #define FOLLY_SV_PACK_PUSH FOLLY_PACK_PUSH
 #define FOLLY_SV_PACK_POP FOLLY_PACK_POP
@@ -1220,8 +1220,9 @@ class small_vector : public detail::small_vector_base<
         heapifyCapacity ? kHeapifyCapacitySize : 0;
     const size_t goodAllocationSizeBytes =
         goodMallocSize(needBytes + allocationExtraBytes);
-    const size_t newCapacity =
+    const size_t goodAllocationNewCapacity =
         (goodAllocationSizeBytes - allocationExtraBytes) / sizeof(value_type);
+    const size_t newCapacity = std::min(goodAllocationNewCapacity, max_size());
     // Make sure that the allocation request has a size computable from the
     // capacity, instead of using goodAllocationSizeBytes, so that we can do
     // sized deallocation. If goodMallocSize() gives us extra bytes that are not
@@ -1417,7 +1418,7 @@ template <
     class C,
     class Predicate>
 void erase_if(small_vector<T, MaxInline, A, B, C>& v, Predicate predicate) {
-  v.erase(std::remove_if(v.begin(), v.end(), predicate), v.end());
+  v.erase(std::remove_if(v.begin(), v.end(), std::ref(predicate)), v.end());
 }
 
 //////////////////////////////////////////////////////////////////////
