@@ -22,7 +22,7 @@
 #include <folly/Memory.h>
 #include <folly/Portability.h>
 #include <folly/executors/QueuedImmediateExecutor.h>
-#include <folly/synchronization/AsymmetricMemoryBarrier.h>
+#include <folly/synchronization/AsymmetricThreadFence.h>
 #include <folly/synchronization/Hazptr-fwd.h>
 #include <folly/synchronization/HazptrObj.h>
 #include <folly/synchronization/HazptrRec.h>
@@ -332,7 +332,8 @@ class hazptr_domain {
     }
     uintptr_t btag = l.head()->cohort_tag();
     bool tagged = ((btag & kTagBit) == kTagBit);
-    /*** Full fence ***/ asymmetricLightBarrier();
+    /*** Full fence ***/ asymmetric_thread_fence_light(
+        std::memory_order_seq_cst);
     List ll(l.head(), l.tail());
     if (!tagged) {
       untagged_[calc_shard(l.head())].push(ll, RetiredList::kMayNotBeLocked);
@@ -383,7 +384,12 @@ class hazptr_domain {
     auto due = load_due_time();
     if (time < due || !cas_due_time(due, time + kSyncTimePeriod))
       return 0;
-    return exchange_count(0);
+    int rcount = exchange_count(0);
+    if (rcount < 0) {
+      add_count(rcount);
+      return 0;
+    }
+    return rcount;
   }
 
   /** check_count_threshold */
@@ -507,12 +513,14 @@ class hazptr_domain {
 
   /** do_reclamation */
   void do_reclamation(int rcount) {
+    DCHECK_GE(rcount, 0);
     while (true) {
       Obj* untagged[kNumShards];
       Obj* tagged[kNumShards];
       bool done = true;
       if (extract_retired_objects(untagged, tagged)) {
-        /*** Full fence ***/ asymmetricHeavyBarrier(AMBFlags::EXPEDITED);
+        /*** Full fence ***/ asymmetric_thread_fence_heavy(
+            std::memory_order_seq_cst);
         Set hs = load_hazptr_vals();
         rcount -= match_tagged(tagged, hs);
         rcount -= match_reclaim_untagged(untagged, hs, done);
