@@ -16,6 +16,7 @@
 
 #include <folly/json.h>
 
+#include <cstdint>
 #include <iterator>
 #include <limits>
 
@@ -348,6 +349,53 @@ TEST(Json, TestLineNumbers) {
   map.clear();
 }
 
+TEST(Json, DuplicateKeys) {
+  dynamic obj = dynamic::object("a", 1);
+  EXPECT_EQ(obj, parseJson("{\"a\": 1}"));
+
+  // Default behavior keeps *last* value.
+  EXPECT_EQ(obj, parseJson("{\"a\": 2, \"a\": 1}"));
+  EXPECT_THROW(
+      parseJson("{\"a\": 2, \"a\": 1}", {.validate_keys = true}), parse_error);
+}
+
+TEST(Json, ParseConvertInt) {
+  EXPECT_THROW(parseJson("{2: 4}"), parse_error);
+  dynamic obj = dynamic::object("2", 4);
+  EXPECT_EQ(obj, parseJson("{2: 4}", {.convert_int_keys = true}));
+  EXPECT_THROW(
+      parseJson("{2: 4, \"2\": 5}", {.convert_int_keys = true}), parse_error);
+  EXPECT_THROW(
+      parseJson("{2: 4, 2: 5}", {.convert_int_keys = true}), parse_error);
+}
+
+TEST(Json, PrintConvertInt) {
+  dynamic obj = dynamic::object(2, 4);
+  EXPECT_THROW(toJson(obj), print_error);
+  EXPECT_EQ(
+      "{2:4}", folly::json::serialize(obj, {.allow_non_string_keys = true}));
+  EXPECT_EQ(
+      "{\"2\":4}", folly::json::serialize(obj, {.convert_int_keys = true}));
+  EXPECT_EQ(
+      "{\"2\":4}",
+      folly::json::serialize(
+          obj,
+          {
+              .allow_non_string_keys = true,
+              .convert_int_keys = true,
+          }));
+
+  obj["2"] = 5; // Would lead to duplicate keys in output JSON.
+  EXPECT_THROW(
+      folly::json::serialize(
+          obj,
+          {
+              .allow_non_string_keys = true,
+              .convert_int_keys = true,
+          }),
+      print_error);
+}
+
 TEST(Json, ParseTrailingComma) {
   folly::json::serialization_opts on, off;
   on.allow_trailing_comma = true;
@@ -695,7 +743,7 @@ TEST(Json, ParseDoubleFallback) {
   EXPECT_EQ(
       std::numeric_limits<int64_t>::max(),
       parseJson("{\"a\":9223372036854775807}").items().begin()->second.asInt());
-  // with double_fallback
+  // with double_fallback numbers outside int64_t range are parsed as double
   folly::json::serialization_opts opts;
   opts.double_fallback = true;
   EXPECT_EQ(
@@ -722,6 +770,38 @@ TEST(Json, ParseDoubleFallback) {
           .items()
           .begin()
           ->second.asDouble());
+  // show that some precision gets lost
+  EXPECT_EQ(
+      847605071342477612345678900000.0,
+      parseJson("{\"a\":847605071342477612345678912345}", opts)
+          .items()
+          .begin()
+          ->second.asDouble());
+  EXPECT_DOUBLE_EQ(
+      -9223372036854775809.0,
+      parseJson("{\"a\":-9223372036854775809}", opts) // first smaller than min
+          .items()
+          .begin()
+          ->second.asDouble());
+  EXPECT_DOUBLE_EQ(
+      9223372036854775808.0,
+      parseJson("{\"a\":9223372036854775808}", opts) // first larger than max
+          .items()
+          .begin()
+          ->second.asDouble());
+  EXPECT_DOUBLE_EQ(
+      -10000000000000000000.0,
+      parseJson("{\"a\":-10000000000000000000}", opts) // minus + 20 digits
+          .items()
+          .begin()
+          ->second.asDouble());
+  EXPECT_DOUBLE_EQ(
+      10000000000000000000.0,
+      parseJson("{\"a\":10000000000000000000}", opts) // 20 digits
+          .items()
+          .begin()
+          ->second.asDouble());
+  // numbers within int64_t range are deserialized as int64_t, no loss
   EXPECT_EQ(
       std::numeric_limits<int64_t>::min(),
       parseJson("{\"a\":-9223372036854775808}", opts)
@@ -734,16 +814,48 @@ TEST(Json, ParseDoubleFallback) {
           .items()
           .begin()
           ->second.asInt());
-  // show that some precision gets lost
   EXPECT_EQ(
-      847605071342477612345678900000.0,
-      parseJson("{\"a\":847605071342477612345678912345}", opts)
+      INT64_C(-1234567890123456789),
+      parseJson("{\"a\":-1234567890123456789}", opts) // minus + 19 digits
           .items()
           .begin()
-          ->second.asDouble());
+          ->second.asInt());
+  EXPECT_EQ(
+      INT64_C(1234567890123456789),
+      parseJson("{\"a\":1234567890123456789}", opts) // 19 digits
+          .items()
+          .begin()
+          ->second.asInt());
+  EXPECT_EQ(
+      INT64_C(-123456789012345678),
+      parseJson("{\"a\":-123456789012345678}", opts) // minus + 18 digits
+          .items()
+          .begin()
+          ->second.asInt());
+  EXPECT_EQ(
+      INT64_C(123456789012345678),
+      parseJson("{\"a\":123456789012345678}", opts) // 18 digits
+          .items()
+          .begin()
+          ->second.asInt());
   EXPECT_EQ(
       toJson(parseJson(R"({"a":-9223372036854775808})", opts)),
       R"({"a":-9223372036854775808})");
+  EXPECT_EQ(
+      toJson(parseJson(R"({"a":9223372036854775807})", opts)),
+      R"({"a":9223372036854775807})");
+  EXPECT_EQ(
+      toJson(parseJson(R"({"a":-1234567890123456789})", opts)),
+      R"({"a":-1234567890123456789})");
+  EXPECT_EQ(
+      toJson(parseJson(R"({"a":1234567890123456789})", opts)),
+      R"({"a":1234567890123456789})");
+  EXPECT_EQ(
+      toJson(parseJson(R"({"a":-123456789012345678})", opts)),
+      R"({"a":-123456789012345678})");
+  EXPECT_EQ(
+      toJson(parseJson(R"({"a":123456789012345678})", opts)),
+      R"({"a":123456789012345678})");
 }
 
 TEST(Json, ParseNumbersAsStrings) {
