@@ -36,7 +36,7 @@
 using folly::small_vector;
 using namespace folly::small_vector_policy;
 
-#if FOLLY_X64 || FOLLY_PPC64
+#if FOLLY_X64 || FOLLY_PPC64 || FOLLY_AARCH64
 
 // Fast access. Heap only small vector
 static_assert(
@@ -1419,3 +1419,15 @@ TEST(small_vector, MoveAndCopyNonTrivial) {
   testMoveAndCopy<NonTrivialInt, 0>();
   testMoveAndCopy<NonTrivialInt, 4>();
 }
+
+TEST(small_vector, PolicyMaxSizeExceeded) {
+  struct Obj {
+    char a[350];
+  };
+
+  small_vector<Obj, 10, uint8_t> v;
+
+  EXPECT_THROW(
+      for (size_t i = 0; i < 0x100; ++i) { v.push_back(Obj()); },
+      std::length_error);
+}
