@@ -641,6 +641,19 @@ TEST(SortedVectorTypes, EraseTest2) {
   EXPECT_EQ(m.size(), 5);
 }
 
+TEST(SortedVectorTypes, EraseIfTest) {
+  sorted_vector_set<int> s1{1, 2, 3, 4, 5, 6, 7, 8, 9};
+  EXPECT_EQ(erase_if(s1, [](int i) { return i % 3 == 0; }), 3);
+  EXPECT_EQ(s1, sorted_vector_set<int>({1, 2, 4, 5, 7, 8}));
+
+  sorted_vector_map<int, int> m1{{1, 10}, {2, 20}, {3, 30}, {4, 44}};
+  EXPECT_EQ(
+      erase_if(m1, [](const auto& kv) { return kv.second == 10 * kv.first; }),
+      3);
+  EXPECT_EQ(m1.size(), 1);
+  EXPECT_EQ(*m1.begin(), std::make_pair(4, 44));
+}
+
 TEST(SortedVectorTypes, TestSetBulkInsertionSortMerge) {
   auto s = std::vector<int>({6, 4, 8, 2});
 
@@ -715,31 +728,49 @@ TEST(SortedVectorTypes, TestSetBulkInsertionSortNoMerge) {
 }
 
 TEST(SortedVectorTypes, TestSetBulkInsertionNoSortMerge) {
-  auto s = std::vector<int>({6, 4, 8, 2});
+  auto test = [](bool withSortedUnique) {
+    auto s = std::vector<int>({6, 4, 8, 2});
 
-  sorted_vector_set<int> vset(s.begin(), s.end());
-  check_invariant(vset);
+    sorted_vector_set<int> vset(s.begin(), s.end());
+    check_invariant(vset);
 
-  // Add a sorted range that will have to be merged in.
-  s = std::vector<int>({1, 3, 5, 9});
+    // Add a sorted range that will have to be merged in.
+    s = std::vector<int>({1, 2, 3, 5, 9});
 
-  vset.insert(s.begin(), s.end());
-  check_invariant(vset);
-  EXPECT_THAT(vset, testing::ElementsAreArray({1, 2, 3, 4, 5, 6, 8, 9}));
+    if (withSortedUnique) {
+      vset.insert(s.begin(), s.end());
+    } else {
+      vset.insert(folly::sorted_unique, s.begin(), s.end());
+    }
+    check_invariant(vset);
+    EXPECT_THAT(vset, testing::ElementsAreArray({1, 2, 3, 4, 5, 6, 8, 9}));
+  };
+
+  test(false);
+  test(true);
 }
 
 TEST(SortedVectorTypes, TestSetBulkInsertionNoSortNoMerge) {
-  auto s = std::vector<int>({6, 4, 8, 2});
+  auto test = [](bool withSortedUnique) {
+    auto s = std::vector<int>({6, 4, 8, 2});
 
-  sorted_vector_set<int> vset(s.begin(), s.end());
-  check_invariant(vset);
+    sorted_vector_set<int> vset(s.begin(), s.end());
+    check_invariant(vset);
 
-  // Add a sorted range that will not have to be merged in.
-  s = std::vector<int>({21, 22, 23, 24});
+    // Add a sorted range that will not have to be merged in.
+    s = std::vector<int>({21, 22, 23, 24});
 
-  vset.insert(s.begin(), s.end());
-  check_invariant(vset);
-  EXPECT_THAT(vset, testing::ElementsAreArray({2, 4, 6, 8, 21, 22, 23, 24}));
+    if (withSortedUnique) {
+      vset.insert(s.begin(), s.end());
+    } else {
+      vset.insert(folly::sorted_unique, s.begin(), s.end());
+    }
+    check_invariant(vset);
+    EXPECT_THAT(vset, testing::ElementsAreArray({2, 4, 6, 8, 21, 22, 23, 24}));
+  };
+
+  test(false);
+  test(true);
 }
 
 TEST(SortedVectorTypes, TestSetBulkInsertionEmptyRange) {
